searchData={"content_type":"text/markdown","items":[{"doc":"Helper for nested map manipulations.","ref":"AbsintheUtils.Internal.MapHelpers.html","title":"AbsintheUtils.Internal.MapHelpers","type":"module"},{"doc":"Put a value in a tested map, if any of the keys in the keys_path are not found,\nthey will be recursively created.","ref":"AbsintheUtils.Internal.MapHelpers.html#recursive_put_in/3","title":"AbsintheUtils.Internal.MapHelpers.recursive_put_in/3","type":"function"},{"doc":"iex> MapHelpers.recursive_put_in(%{a: 1}, [:b], 2)\n    %{a: 1, b: 2}\n\n    iex> MapHelpers.recursive_put_in(%{}, [:a], 1)\n    %{a: 1}\n\n    iex> MapHelpers.recursive_put_in(%{}, :a, 1)\n    %{a: 1}","ref":"AbsintheUtils.Internal.MapHelpers.html#recursive_put_in/3-examples","title":"Examples - AbsintheUtils.Internal.MapHelpers.recursive_put_in/3","type":"function"},{"doc":"Pop a key from a nested map, if successful returns a tuple of:\n - the popped value\n - the modified map, without the popped key\nOR if the key is not found, returns :error","ref":"AbsintheUtils.Internal.MapHelpers.html#safe_pop_in/2","title":"AbsintheUtils.Internal.MapHelpers.safe_pop_in/2","type":"function"},{"doc":"iex> MapHelpers.safe_pop_in(%{a: 1}, [:a])\n    {1, %{}}\n\n    iex> MapHelpers.safe_pop_in(%{a: 1}, :a)\n    {1, %{}}\n\n    iex> MapHelpers.safe_pop_in(%{a: 1}, [:invalid])\n    :error\n\n    iex> MapHelpers.safe_pop_in(%{a: %{b: 1}}, [:a, :b])\n    {1, %{a: %{}}}\n\n    iex> MapHelpers.safe_pop_in(%{a: %{b: 1}}, [:a, :invalid])\n    :error\n\n    iex> MapHelpers.safe_pop_in(%{}, [:a])\n    :error\n\n    iex> MapHelpers.safe_pop_in(%{a: 1}, [])\n    :error\n\n    iex> MapHelpers.safe_pop_in(%{a: 1}, [:a, :b, :c])\n    :error\n\n    iex> MapHelpers.safe_pop_in(%{a: 1, b: %{c: 2}}, [:b])\n    {%{c: 2}, %{a: 1}}\n\n    iex> MapHelpers.safe_pop_in(%{\"a\" => 1}, [\"a\"])\n    {1, %{}}","ref":"AbsintheUtils.Internal.MapHelpers.html#safe_pop_in/2-examples","title":"Examples - AbsintheUtils.Internal.MapHelpers.safe_pop_in/2","type":"function"},{"doc":"The JSON scalar type allows arbitrary JSON values to be passed in and out.\nRequires `{ :jason, \">= 1.1\" }` package: https://github.com/michalmuskala/jason\n\nBased in the\n[recipes on Absinthe's wiki](https://github.com/absinthe-graphql/absinthe/wiki/Scalar-Recipes)\n\nNote that even if you use `non_null(:json)` a string of null value (`\"null\"`) is still accepted.\n\n**Usage:**\n\nImport the type in your schema `import_types(AbsintheUtils.Scalars.JSON)` and you will be able\nto use the `:json` type.","ref":"AbsintheUtils.Scalars.JSON.html","title":"AbsintheUtils.Scalars.JSON","type":"module"},{"doc":"The `StrictNaiveDateTime` scalar type represents a naive date and time without\ntimezone.\nThe output is an ISO8601 formatted string.\nThe input must be a naive datetime, without timezone offset.\n\nValid examples:\n- 2020-01-01T00:00:00\n- 2020-01-01 00:00:00\n\nInvalid examples:\n- 2020-01-01T00:00:00+00:00\n- 2020-01-01T00:00:00+01:00\n- 2020-01-01T00:00:00Z\n- 2020-01-01 00:00:00Z\n\n\nBased on the type `naive_datetime` from `Absinthe.Type.Custom`.","ref":"AbsintheUtils.Scalars.StrictNaiveDateTime.html","title":"AbsintheUtils.Scalars.StrictNaiveDateTime","type":"module"},{"doc":"The UUID scalar type allows UUID compliant strings to be passed in and out.\nRequires `{ :ecto, \">= 0.0.0\" }` package: https://github.com/elixir-ecto/ecto\n\nBased in the\n[recipes on Absinthe's wiki](https://github.com/absinthe-graphql/absinthe/wiki/Scalar-Recipes)\n\n**Usage:**\n\nImport the type in your schema `import_types(AbsintheUtils.Scalars.UUID)` and you will be able\nto use the `:uuid` type.","ref":"AbsintheUtils.Scalars.UUID.html","title":"AbsintheUtils.Scalars.UUID","type":"module"},{"doc":"Absinthe middleware for loading entities in `field` arguments.\n\nThis middleware should be defined before `resolve`. It will manipulate the arguments\nbefore they are passed to the resolver function.\n\nAs configuration it accepts a map of original argument names (or path) to a keyword list, containing:\n\n- `new_name`: the new key to push the loaded entity into, can be a list of atoms\n  to push the entity into a nested map.\n  (optional, defaults to the argument name - the key of the map configuration).\n- `load_function`: the function used to load the argument into an entity.\n  As an input accepts two arguments:\n  - `context`: the context of the current resolution (prior to any modifications of the current middleware).\n  - `input_value`: the value received in the value of `argument_name`.\n  The function should return the entity or a list of entities.\n  `nil` or an empty list when not found .\n- `nil_is_not_found`: whether to consider `nil` as a not found value.\n  (optional, defaults to `true`).","ref":"AbsintheUtils.Middleware.ArgLoader.html","title":"AbsintheUtils.Middleware.ArgLoader","type":"module"},{"doc":"```\nquery do\n  field :user, :user do\n    arg(:id, :id)\n\n    # Add the middleware before your resolver\n    middleware(\n      ArgLoader,\n      %{\n        id: [\n          new_name: :user,\n          load_function: fn _context, id ->\n            get_user_by_id(id)\n          end,\n          nil_is_not_found: false\n        ]\n      }\n    )\n\n    resolve(fn _, arguments, _ ->\n      {:ok, Map.get(arguments, :user)}\n    end)\n  end\n```\n\nThis will define a `user` query that accepts an `id` input. Before calling the resolver,","ref":"AbsintheUtils.Middleware.ArgLoader.html#module-examples","title":"Examples - AbsintheUtils.Middleware.ArgLoader","type":"module"},{"doc":"`ArgLoader` can also be used to load a `list_of` arguments:\n\n```\nquery do\n  field :users, non_null(list_of(:user)) do\n    arg(:ids, non_null(list_of(:id)))\n\n    middleware(\n      ArgLoader,\n      %{\n        ids: [\n          new_name: :users,\n          load_function: fn _context, ids ->\n            ids\n            |> get_users_by_id()\n            |> AbsintheUtils.Helpers.Sorting.sort_alike(ids, & &1.id)\n          end\n        ]\n      }\n    )\n\n    resolve(fn _, params, _ ->\n      {:ok, Map.get(params, :users)}\n    end)\n  end\nend\n```","ref":"AbsintheUtils.Middleware.ArgLoader.html#module-list-of-entities","title":"List of entities - AbsintheUtils.Middleware.ArgLoader","type":"module"},{"doc":"Using `ArgLoader` to load and rename deep nested arguments:\n\n```\ninput_object :complex_input_object do\n  field(:user_id, :id)\nend\n\nquery do\n  field :users, :boolean do\n    arg(:input, :complex_input_object)\n\n    middleware(\n      ArgLoader,\n      %{\n        [:input, :user_id] => [\n          new_name: [:loaded_entities, :user],\n          load_function: fn _context, user_id ->\n            get_user_by_id(user_id)\n          end\n        ]\n      }\n    )\n\n    resolve(fn _, params, _ ->\n      # params.loaded_entities.user will contain the loaded user\n      {:ok, true}\n    end)\n  end\nend\n```\n\n\nNote the use of `AbsintheUtils.Helpers.Sorting.sort_alike/2` to ensure the returned list of\nentities from the repository is sorted according to the user's input.","ref":"AbsintheUtils.Middleware.ArgLoader.html#module-deep-nested-arguments","title":"Deep nested arguments - AbsintheUtils.Middleware.ArgLoader","type":"module"},{"doc":"Absinthe middleware for handling deprecated or renamed `field` arguments (`arg`).\n\nIt automatically handles mutual exclusivity and changing the argument keys (names)\nbefore passing it to the resolver.\nFor this to work, this middleware must be added before the resolver.\n\nMiddleware options:\n\n - `legacy_arg_identifier`: The identifier of the field that has been deprecated.\n   If this argument is provided, this middleware will rename it to the `new_arg_identifier`.\n\n - `new_arg_identifier`: The identifier of the new field, if passed, nothing changes.\n\n - `is_required`: if at least one of the arguments must be provided.\n   If the validation fails, an Absinthe compliant error will be returned and\n   the operation marked as resolved (it will not reach the resolver).\n\nAll options are required.","ref":"AbsintheUtils.Middleware.DeprecatedArgs.html","title":"AbsintheUtils.Middleware.DeprecatedArgs","type":"module"},{"doc":"```\nfield :query_with_deprecated_required_args, non_null(:string) do\n  arg(:old_arg, :string, deprecate: \"Use `newParam` instead.\")\n  arg(:new_arg, :string)\n\n  # Add the middleware before your resolver\n  middleware(\n    DeprecatedArgs,\n    %{\n      legacy_arg_identifier: :old_arg,\n      new_arg_identifier: :new_arg,\n      is_required: true\n    }\n  )\n\n  resolve(&MyApp.my_resolver/3)\nend\n```","ref":"AbsintheUtils.Middleware.DeprecatedArgs.html#module-example-usage","title":"Example usage - AbsintheUtils.Middleware.DeprecatedArgs","type":"module"},{"doc":"Helpers for Absinthe errors.","ref":"AbsintheUtils.Helpers.Errors.html","title":"AbsintheUtils.Helpers.Errors","type":"module"},{"doc":"Given an absinthe resolution puts an error and marks the resolution as resolved.","ref":"AbsintheUtils.Helpers.Errors.html#put_error/3","title":"AbsintheUtils.Helpers.Errors.put_error/3","type":"function"},{"doc":"Generic sorting utils.","ref":"AbsintheUtils.Helpers.Sorting.html","title":"AbsintheUtils.Helpers.Sorting","type":"module"},{"doc":"Sorts an `unsorted_enumerable` based on `sorted_enumerable`.\nUse mappers to specify getters for each element in the enumerable,\nfor example retrieving the id of a struct.","ref":"AbsintheUtils.Helpers.Sorting.html#sort_alike/4","title":"AbsintheUtils.Helpers.Sorting.sort_alike/4","type":"function"},{"doc":"iex> Sorting.sort_alike([:b, :c, :a], [:a, :b, :c])\n    [:a, :b, :c]\n\n    iex> Sorting.sort_alike([%{id: 1}, %{id: 2}], [2, 1], & &1.id)\n    [%{id: 2}, %{id: 1}]","ref":"AbsintheUtils.Helpers.Sorting.html#sort_alike/4-examples","title":"Examples - AbsintheUtils.Helpers.Sorting.sort_alike/4","type":"function"},{"doc":"# Absinthe Utils\n\nCollection of utilities for [absinthe](https://hexdocs.pm/absinthe).\n\n[![absinthe_utils in hex.pm](https://img.shields.io/hexpm/v/absinthe_utils?style=flat)][hexpm]\n[![absinthe_utils documentation](https://img.shields.io/badge/hex.pm-docs-green.svg?style=flat)][hexdocs]","ref":"readme.html","title":"Absinthe Utils","type":"extras"},{"doc":"[Available in Hex][hexdocs], the package can be installed\nby adding `absinthe_utils` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    # Check the releases page for the desired version (and use sigils accordingly, ex: \"~>\").\n    {:absinthe_utils, \">= 0.0.0\"}\n  ]\nend\n```\n\nDocumentation can be found in [HexDocs].\n\n# Main features\n\n- `AbsintheUtils.Middleware.ArgLoader`: Middleware for loading entities in `field` arguments.\n- `AbsintheUtils.Middleware.DeprecatedArgs`: Middleware for handling deprecated or renamed `field`\n  arguments (`arg`).\n- `AbsintheUtils.Scalars.JSON`: JSON scalar.\n- `AbsintheUtils.Scalars.UUID`: UUID scalar.\n- `AbsintheUtils.Scalars.StrictNaiveDateTime`: NaiveDatetime that does not accept ISO8601 with offset.\n\n[hexpm]: https://hex.pm/packages/absinthe_utils\n[hexdocs]: https://hexdocs.pm/absinthe_utils","ref":"readme.html#installation","title":"Installation - Absinthe Utils","type":"extras"}],"producer":{"name":"ex_doc","version":"0.37.3"},"proglang":"elixir"}